//+------------------------------------------------------------------+
//|                                                     VolumeMA.mq5 |
//|                                           Copyright 2022, ernst. |
//|                                   https://www.mql5.com/en/pippod |
//+------------------------------------------------------------------+
#property copyright "Copyright 2022, ernst."
#property link      "https://www.mql5.com/en/pippod"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_plots   2
//--- plot Volume
#property indicator_label1  "Volume"
#property indicator_type1   DRAW_COLOR_HISTOGRAM
#property indicator_color1  clrDodgerBlue,clrFireBrick,clrYellow
#property indicator_style1  STYLE_SOLID
#property indicator_width1  5
//--- plot MA
#property indicator_label2  "MA"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrLimeGreen
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//---
#property indicator_minimum 0
//--- input parameters
input int      Period=14;
input ENUM_MA_METHOD Method=MODE_LWMA;
//--- indicator buffers
double         VolumeBuffer[];
double         VolumeColors[];
double         MABuffer[];
#include <MovingAverages.mqh>
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   IndicatorSetString(INDICATOR_SHORTNAME,StringFormat("Volume %s(%d)",StringSubstr(EnumToString(Method),5),Period));
   IndicatorSetInteger(INDICATOR_DIGITS,0);
//--- indicator buffers mapping
   SetIndexBuffer(0,VolumeBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,VolumeColors,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(2,MABuffer,INDICATOR_DATA);
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int begin=(rates_total==prev_calculated)?prev_calculated-1:prev_calculated;
   for(int i=begin;i<rates_total;i++)
      VolumeBuffer[i]=(double)tick_volume[i];
//---
   switch(Method) {
      case MODE_SMA:
         SimpleMAOnBuffer(rates_total,prev_calculated,0,Period,VolumeBuffer,MABuffer);
         break;
      case MODE_EMA:
         ExponentialMAOnBuffer(rates_total,prev_calculated,0,Period,VolumeBuffer,MABuffer);
         break;
      case MODE_SMMA:
         SmoothedMAOnBuffer(rates_total,prev_calculated,0,Period,VolumeBuffer,MABuffer);
         break;
      case MODE_LWMA:
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,Period,VolumeBuffer,MABuffer,sumPrice,sumWeight);
         break; }
//---
   if(!prev_calculated)
      begin=Period-1;
   for(int i=begin;i<rates_total;i++)
      VolumeColors[i]=(VolumeBuffer[i]>MABuffer[i])?(close[i]>open[i])?0:1:2;
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int sumWeight;
double sumPrice;
int LinearWeightedMAOnBuffer(const int rates_total,
                             const int prev_calculated,
                             const int begin,
                             const int period,
                             const double& price[],
                             double& buffer[],
                             double& sum_price,
                             int& sum_weight)
  {
//--- check new bar
   bool newBar=rates_total!=prev_calculated;
   int i,k,limit=(newBar)?prev_calculated:prev_calculated-1;
//--- save as_series flags
   bool as_series_price=ArrayGetAsSeries(price);
   bool as_series_buffer=ArrayGetAsSeries(buffer);
   if(as_series_price) ArraySetAsSeries(price,false);
   if(as_series_buffer) ArraySetAsSeries(buffer,false);
//--- first calculation or number of bars has changed
   if(prev_calculated==0) {
//--- check for data
      if(period<=1 || (limit=begin+period-1)>=rates_total) 
         return(0);
      for(i=0;i<limit;i++)
         buffer[i]=EMPTY_VALUE;
//--- calculate first visible value
      sum_price=sum_weight=0;
      for(i=begin,k=1;i<=limit;i++,k++) {
         sum_price+=k*price[i];
         sum_weight+=k; }
//--- set first value
      buffer[limit++]=sum_price/sum_weight; }
//--- main loop
   for(i=limit;i<rates_total;i++) {
//--- get price sum on new bar
      if(newBar) {
         sum_price=0;
         for(k=1;k<period;k++) 
            sum_price+=price[i-period+k]*k; }
//--- get last bar's value
      buffer[i]=(sum_price+price[i]*period)/sum_weight; }
//--- restore as_series flags
   if(as_series_price) ArraySetAsSeries(price,true);
   if(as_series_buffer) ArraySetAsSeries(buffer,true);
//---
   return(rates_total);
  }
//+------------------------------------------------------------------+
